use chrono::{DateTime, Local, NaiveDate, NaiveDateTime, TimeZone};
use eyre::bail;
use std::str::FromStr;

pub struct HumanDateTime(pub DateTime<Local>);

impl FromStr for HumanDateTime {
    type Err = eyre::Error;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        if let Ok(date_only) = NaiveDate::parse_from_str(s, "%Y-%m-%d") {
            let local_datetime = Local
                .from_local_datetime(&date_only.and_hms_opt(0, 0, 0).unwrap())
                .unwrap();
            Ok(HumanDateTime(local_datetime))
        } else if let Ok(date_and_time) = NaiveDateTime::parse_from_str(s, "%Y-%m-%dT%H:%M:%S") {
            let local_datetime = Local.from_local_datetime(&date_and_time).unwrap();
            Ok(HumanDateTime(local_datetime))
        } else if let Ok(date_and_time) = NaiveDateTime::parse_from_str(s, "%Y-%m-%d %H:%M:%S") {
            let local_datetime = Local.from_local_datetime(&date_and_time).unwrap();
            Ok(HumanDateTime(local_datetime))
        } else {
            bail!("Couldn't parse using any format. Use one of: 2021-05-16 OR 2021-05-16T17:42:14 OR 2021-05-16 17:42:14");
        }
    }
}
