use chrono::{DateTime, NaiveDateTime, TimeZone, Utc};

pub const POINTER_DATETIME_FORMAT: &'static str = "%F_%T";
pub const POINTER_NAME_DATETIME_SPLITTER: &'static str = "+";

pub fn get_pointer_name_at(source_name: &str, datetime: DateTime<Utc>) -> String {
    format!(
        "{}{}{}",
        source_name,
        POINTER_NAME_DATETIME_SPLITTER,
        datetime.format(POINTER_DATETIME_FORMAT).to_string()
    )
}

pub fn split_pointer_name(pointer_name: &str) -> Option<(String, DateTime<Utc>)> {
    let (source_name, date_time_str) = pointer_name.rsplit_once(POINTER_NAME_DATETIME_SPLITTER)?;
    let date_time = NaiveDateTime::parse_from_str(date_time_str, POINTER_DATETIME_FORMAT).ok()?;
    let date_time = Utc.from_utc_datetime(&date_time);
    Some((source_name.to_owned(), date_time))
}
