# Storage Mechanisms ('Raw Piles')

The basic storage layer abstraction is called a Raw Pile. There are a few implementations of these, including a few which wrap other Raw Piles to enhance them with features such as compression or encryption.

## Storage

These Raw Pile implementations store data locally.

### SQLite-Indexed Bloblog Raw Pile

This Raw Pile stores data on disk.

Chunk contents are written into 'blob logs'. The position of a chunk's contents is then stored in a SQLite index, to allow much faster random read time to the chunk.


## Transformers

These Raw Pile implementations accept another Raw Pile and transform the data passing through.

### Compression Raw Pile

This Raw Pile augments another Raw Pile with Zstd compression. Compression and decompression is done in worker threads.

### Encryption Raw Pile

This Raw Pile augments another Raw Pile with `crypto_secretbox_xsalsa20poly1305` encryption from NaCl.

/!\ Encryption is currently NOT CPA secure, so an adversary may be able to detect what files you have. (This is because chunk hashes are not yet encrypted).

**THIS IS NOT FIT FOR USE AND NO COMPATIBILITY WILL BE MAINTAINED IN FUTURE YAMA VERSIONS.**

### Integrity Raw Pile

This Raw Pile augments another Raw Pile with XxHash64 checksums, to allow fast integrity checking. (An alternative might be to have this at the filesystem level e.g. with Btrfs or ZFS.)

This is not strictly necessary since the Chunk IDs themselves are a hash of the contents. However, use of this Integrity Raw Pile can allow checking integrity without needing to perform expensive decompression and decryption, or it can allow a server to check the integrity without sending everything to the client for decryption.


## Remote Raw Piles

These EXPERIMENTAL Raw Piles allow access to another Raw Pile over the network.

### Requester Raw Pile

This Raw Pile allows you to open an SSH connection (to a *Responder*) and access a Raw Pile on the *Responder* remotely.

TODO but probably will want to stack things as follows:

* Compression
* Encryption
* Requester (SSH connection)
* Integrity
* SQLite-indexed Bloblog

This order provides compression (reduced disk usage), encryption (the server can't see the client's data) and server-side integrity (so the server can quickly check that it has not corrupted things).
