use tokio::signal::unix::SignalKind;
use tracing::warn;

/// Registers a signal handler on SIGUSR1 that dumps a backtrace of the tokio task tree.
///
/// May be useful for debugging deadlocks etc.
pub fn register_sigusr1_backtrace_helper() {
    tokio::spawn(async {
        while let Some(()) = tokio::signal::unix::signal(SignalKind::user_defined1())
            .unwrap()
            .recv()
            .await
        {
            warn!(
                "SIGUSR1 received; debug task backtrace:\n{}",
                async_backtrace::taskdump_tree(false)
            );
        }
    });
}
