use crate::byte_layer::ByteLayer;
use eyre::Context;
use std::marker::PhantomData;

pub struct Zstd<T> {
    bytes: Vec<u8>,
    marker: PhantomData<T>,
}

impl<T> ByteLayer for Zstd<T> {
    fn from_byte_vec(bytes: Vec<u8>) -> Self {
        Self {
            bytes,
            marker: Default::default(),
        }
    }

    fn into_byte_vec(self) -> Vec<u8> {
        self.bytes
    }
}

pub const OUR_DEFAULT_COMPRESSION_LEVEL: i32 = 10;

impl<T: ByteLayer> Zstd<T> {
    pub fn compress(src: T) -> Self {
        let bytes = zstd::encode_all(&src.into_byte_vec()[..], OUR_DEFAULT_COMPRESSION_LEVEL)
            .expect("zstd shouldn't fail to compress");
        Self {
            bytes,
            marker: Default::default(),
        }
    }

    pub fn decompress(self) -> eyre::Result<T> {
        let decompressed_bytes =
            zstd::decode_all(&self.bytes[..]).context("zstd decompression failure")?;
        Ok(T::from_byte_vec(decompressed_bytes))
    }
}
