use crate::tree::{FilesystemOwnership, FilesystemPermissions};
use serde::{Deserialize, Serialize};

/// A tree node, but unpopulated and not a tree.
#[derive(Debug, Clone, Serialize, Deserialize, Eq, PartialEq)]
pub enum ScanEntry {
    NormalFile {
        /// modification time in ms
        mtime: u64,
        #[serde(flatten)]
        ownership: FilesystemOwnership,
        #[serde(flatten)]
        permissions: FilesystemPermissions,
        size: u64,
    },
    Directory {
        #[serde(flatten)]
        ownership: FilesystemOwnership,
        #[serde(flatten)]
        permissions: FilesystemPermissions,
    },
    SymbolicLink {
        #[serde(flatten)]
        ownership: FilesystemOwnership,
        target: String,
    },
}
